package org.wheel.widget;

import java.util.ArrayList;
import java.util.Calendar;
import java.util.Locale;

import com.wheel.libary.R;

import android.content.Context;
import android.util.AttributeSet;
import android.view.LayoutInflater;
import android.widget.LinearLayout;

public class WheelTimePicker extends LinearLayout {

	private Context mContext;
	protected static final int UPDATE_WHEEL = 0x112;
	protected static final int UPDATE_UpdateDay_MSG = 0x113;
	private final int START_YEAR = 1970;
	private final int END_YEAR = 2100;
	private WheelView mWheelYear;
	private WheelView mWheelMonth;
	private WheelView mWheelDay;
	protected int mYear;
	protected int mMonth;
	protected int mDay;

	private Calendar mCalendar;
	private int mDefaultDayWhellIndex = 0;
	private WheelHandler mHandler;
	private WheelView.OnSelectListener mYearListener = new WheelView.OnSelectListener() {
		@Override
		public void endSelect(int year, String text) {
			mYear = START_YEAR + year;
		}

		@Override
		public void selecting(int id, String text) {
		}
	};

	private WheelView.OnSelectListener mMonthListener = new WheelView.OnSelectListener() {
		@Override
		public void endSelect(int month, String text) {
			mMonth = month;
			mHandler.sendEmptyMessage(UPDATE_UpdateDay_MSG);
		}

		@Override
		public void selecting(int id, String text) {
		}
	};

	private WheelView.OnSelectListener mDayListener = new WheelView.OnSelectListener() {
		@Override
		public void endSelect(int day, String text) {
			mDay = day + 1;
		}

		@Override
		public void selecting(int day, String text) {
		}
	};

	public WheelTimePicker(Context context) {
		this(context, null);
	}

	public WheelTimePicker(Context context, AttributeSet attrs) {
		super(context, attrs);
	}

	@Override
	protected void onFinishInflate() {
		super.onFinishInflate();
		mHandler = new WheelHandler(this);
		mContext = getContext();
		LayoutInflater.from(mContext).inflate(R.layout.time_picker, this);
		mWheelYear = (WheelView) findViewById(R.id.year);
		mWheelMonth = (WheelView) findViewById(R.id.month);
		mWheelDay = (WheelView) findViewById(R.id.day);
		mWheelYear.setOnSelectListener(mYearListener);
		mWheelMonth.setOnSelectListener(mMonthListener);
		mWheelDay.setOnSelectListener(mDayListener);
	}

	protected void updateDay(int month) {
		Calendar calendar = Calendar.getInstance();
		calendar.set(Calendar.MONTH, month);
		int maxDay = calendar.getActualMaximum(Calendar.DAY_OF_MONTH);
		mWheelDay.resetData(getDayData(maxDay));
		if (mDay > maxDay) {
			mWheelDay.setDefault(mDefaultDayWhellIndex);
			mDay = mDefaultDayWhellIndex + 1;
		} else {
			mWheelDay.setDefault(mDay - 1);
		}
	}

	/**
	 * set date
	 * @param date
	 */
	public void setDate(long date) {
		mCalendar = Calendar.getInstance(Locale.CHINA);
		mCalendar.setTimeInMillis(date);
		mYear = mCalendar.get(Calendar.YEAR);
		mMonth = mCalendar.get(Calendar.MONTH);
		mDay = mCalendar.get(Calendar.DAY_OF_MONTH);

		mWheelYear.setData(getYearData());
		mWheelMonth.setData(getMonthData());
		mWheelDay.setData(getDayData(mCalendar
				.getActualMaximum(Calendar.DAY_OF_MONTH)));

		mHandler.sendEmptyMessage(UPDATE_WHEEL);
	}

	protected void updateWheel() {
		mWheelYear.setDefault(mYear - START_YEAR);
		mWheelMonth.setDefault(mMonth);
		mWheelDay.setDefault(mDay - 1);
	}

	private ArrayList<String> getYearData() {
		ArrayList<String> list = new ArrayList<String>();
		for (int i = START_YEAR; i <= END_YEAR; i++) {
			list.add(i + "");
		}
		return list;
	}

	private ArrayList<String> getMonthData() {
		ArrayList<String> list = new ArrayList<String>();
		for (int i = 1; i <= 12; i++) {
			list.add(i + "");
		}
		return list;
	}

	private ArrayList<String> getDayData(int endDay) {
		ArrayList<String> list = new ArrayList<String>();
		for (int i = 1; i <= endDay; i++) {
			list.add(i + "");
		}
		return list;
	}

}