package org.wheel.widget;

import java.util.ArrayList;
import java.util.concurrent.ExecutorService;
import java.util.concurrent.Executors;

import android.annotation.SuppressLint;
import android.content.Context;
import android.content.res.TypedArray;
import android.graphics.Canvas;
import android.graphics.LinearGradient;
import android.graphics.Paint;
import android.graphics.Rect;
import android.graphics.Shader.TileMode;
import android.os.Handler;
import android.util.AttributeSet;
import android.view.MotionEvent;
import android.view.View;

import com.wheel.libary.R;

/**
 * WheelView
 */
@SuppressLint("ClickableViewAccessibility")
public class WheelView extends View {
	/**
	 * ؼ
	 */
	private float controlWidth;
	/**
	 * ؼ߶
	 */
	private float controlHeight;
	/**
	 * Ƿ񻬶
	 */
	private boolean isScrolling = false;
	/**
	 * ѡ
	 */
	private ArrayList<ItemObject> itemList = new ArrayList<ItemObject>();
	/**
	 * 
	 */
	private ArrayList<String> dataList = new ArrayList<String>();
	/**
	 * µ
	 */
	private int downY;
	/**
	 * µʱ
	 */
	private long downTime = 0;
	/**
	 * ̴ƶ
	 */
	private long goonTime = 50;
	/**
	 * ̴ƶ
	 */
	private int goonDistence = 100;
	/**
	 * ߻
	 */
	private Paint linePaint;
	/**
	 * ײ߻
	 */
	private Paint bottomlinePaint;
	/**
	 * ײָߵļ
	 */
	float firstLineAndSecondLineSpace = 50;
	/**
	 * ߵĬɫ
	 */
	private int lineColor = 0xffff0000;
	/**
	 * ߵĬɫ
	 */
	private int bottomLineColor = 0xffDADADA;
	/**
	 * ߵĬϿ
	 */
	private float lineWidth = 2f;
	/**
	 * Ĭ
	 */
	private float normalFont = 10.0f;
	/**
	 * ѡеʱ
	 */
	private float selectedFont = 16.0f;
	/**
	 * Ԫ߶
	 */
	private int unitHeight = 50;
	/**
	 * ʾٸ
	 */
	private int itemNumber = 7;
	/**
	 * Ĭɫ
	 */
	private int normalColor = 0xff000000;
	/**
	 * ѡʱɫ
	 */
	private int selectedColor = 0xffff0000;
	/**
	 * ɰ߶
	 */
	private float maskHight = 48.0f;
	/**
	 * ѡ
	 */
	private OnSelectListener onSelectListener;
	/**
	 * Ƿ
	 */
	private boolean isEnable = true;
	/**
	 * ƶ
	 */
	private static final int MOVE_NUMBER = 5;
	/**
	 * Ƿѡ
	 */
	private boolean noEmpty = true;

	/**
	 * ޸ݣConcurrentModificationException쳣
	 */
	private boolean isClearing = false;
	private ExecutorService cachedThreadPool;
	private Handler mHandler = new Handler();

	public WheelView(Context context, AttributeSet attrs, int defStyle) {
		super(context, attrs, defStyle);
		init(context, attrs);
		initData();
	}

	public WheelView(Context context, AttributeSet attrs) {
		super(context, attrs);
		init(context, attrs);
		initData();
	}

	public WheelView(Context context) {
		super(context);
		initData();
	}

	@Override
	public boolean onTouchEvent(MotionEvent event) {
		if (!isEnable)
			return true;
		int y = (int) event.getY();
		switch (event.getAction()) {
		case MotionEvent.ACTION_DOWN:
			isScrolling = true;
			downY = (int) event.getY();
			downTime = System.currentTimeMillis();
			break;
		case MotionEvent.ACTION_MOVE:
			actionMove(y - downY);
			onSelectListener();
			break;
		case MotionEvent.ACTION_UP:
			int move = Math.abs(y - downY);
			// ж϶ʱƶľ
			if (System.currentTimeMillis() - downTime < goonTime
					&& move > goonDistence) {
				goonMove(y - downY);
			} else {
				actionUp(y - downY);
			}
			noEmpty();
			isScrolling = false;
			break;
		default:
			break;
		}
		return true;
	}

	@Override
	protected void onDraw(Canvas canvas) {
		super.onDraw(canvas);
		drawLine(canvas);
		drawList(canvas);
		drawMask(canvas);
	}

	private synchronized void drawList(Canvas canvas) {
		if (isClearing)
			return;
		try {
			for (ItemObject itemObject : itemList) {
				itemObject.drawSelf(canvas);
			}
		} catch (Exception e) {
		}

	}

	@Override
	protected void onLayout(boolean changed, int left, int top, int right,
			int bottom) {
		super.onLayout(changed, left, top, right, bottom);
	}

	@Override
	protected void onMeasure(int widthMeasureSpec, int heightMeasureSpec) {
		super.onMeasure(widthMeasureSpec, heightMeasureSpec);
		controlWidth = getWidth();
		if (controlWidth != 0) {
			setMeasuredDimension(getWidth(), itemNumber * unitHeight);
			controlWidth = getWidth();
		}

	}

	/**
	 * ƶһ
	 */
	private synchronized void goonMove(final int move) {
		cachedThreadPool.execute(new Runnable() {

			@Override
			public void run() {
				int distence = 0;
				while (distence < unitHeight * MOVE_NUMBER) {
					sleep(5);
					actionThreadMove(move > 0 ? distence : distence * (-1));
					distence += 10;
				}
				actionUp(move > 0 ? distence - 10 : distence * (-1) + 10);
				noEmpty();
			}
		});
	}

	/**
	 * Ϊգѡ
	 */
	private void noEmpty() {
		if (!noEmpty)
			return;
		for (ItemObject item : itemList) {
			if (item.isSelected())
				return;
		}
		int move = (int) itemList.get(0).moveToSelected();
		if (move < 0) {
			defaultMove(move);
		} else {
			defaultMove((int) itemList.get(itemList.size() - 1)
					.moveToSelected());
		}
		for (ItemObject item : itemList) {
			if (item.isSelected()) {
				if (onSelectListener != null)
					onSelectListener.endSelect(item.id, item.itemText);
				break;
			}
		}
	}

	/**
	 * ʼ
	 */
	private void initData() {
		isClearing = true;
		itemList.clear();
		for (int i = 0; i < dataList.size(); i++) {
			ItemObject itmItemObject = new ItemObject();
			itmItemObject.id = i;
			itmItemObject.itemText = dataList.get(i);
			itmItemObject.x = 0;
			itmItemObject.y = i * unitHeight;
			itemList.add(itmItemObject);
		}
		isClearing = false;
	}

	/**
	 * ƶʱ
	 * 
	 * @param move
	 */
	private void actionMove(int move) {
		for (ItemObject item : itemList) {
			item.move(move);
		}
		invalidate();
	}

	/**
	 * ƶ߳е
	 * 
	 * @param move
	 */
	private void actionThreadMove(int move) {
		for (ItemObject item : itemList) {
			item.move(move);
		}
		updateView();
	}

	/**
	 * ɿʱ
	 * 
	 * @param move
	 */
	private void actionUp(int move) {
		int newMove = 0;
		if (move > 0) {
			for (int i = 0; i < itemList.size(); i++) {
				if (itemList.get(i).isSelected()) {
					newMove = (int) itemList.get(i).moveToSelected();
					if (onSelectListener != null)
						onSelectListener.endSelect(itemList.get(i).id,
								itemList.get(i).itemText);
					break;
				}
			}
		} else {
			for (int i = itemList.size() - 1; i >= 0; i--) {
				if (itemList.get(i).isSelected()) {
					newMove = (int) itemList.get(i).moveToSelected();
					if (onSelectListener != null)
						onSelectListener.endSelect(itemList.get(i).id,
								itemList.get(i).itemText);
					break;
				}
			}
		}
		for (ItemObject item : itemList) {
			item.newY(move + 0);
		}
		slowMove(newMove);
		updateView();
	}

	/**
	 * ƶ
	 * 
	 * @param move
	 */
	private void slowMove(final int move) {
		cachedThreadPool.execute(new Runnable() {

			@Override
			public void run() {
				// TODO Auto-generated method stub
				int m = move > 0 ? move : move * (-1);
				int i = move > 0 ? 1 : (-1);
				// ƶٶ
				int speed = 1;
				while (true) {
					m = m - speed;
					if (m <= 0) {
						for (ItemObject item : itemList) {
							item.newY(m * i);
						}
						updateView();
						sleep(2);
						break;
					}
					for (ItemObject item : itemList) {
						item.newY(speed * i);
					}
					updateView();
					sleep(2);
				}
				for (ItemObject item : itemList) {
					if (item.isSelected()) {
						if (onSelectListener != null)
							onSelectListener.endSelect(item.id, item.itemText);
						break;
					}
				}
			}
		});
	}

	private void sleep(long time) {
		try {
			Thread.sleep(time);
		} catch (InterruptedException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	/**
	 * ƶĬλ
	 * 
	 * @param move
	 */
	private void defaultMove(int move) {
		for (ItemObject item : itemList) {
			item.newY(move);
		}
		updateView();
	}

	/**
	 * 
	 */
	private void onSelectListener() {
		if (onSelectListener == null)
			return;
		for (ItemObject item : itemList) {
			if (item.isSelected()) {
				onSelectListener.selecting(item.id, item.itemText);
			}
		}
	}

	/**
	 * 
	 * 
	 * @param canvas
	 */
	private void drawLine(Canvas canvas) {

		if (linePaint == null) {
			linePaint = new Paint();
			linePaint.setColor(bottomLineColor);
			linePaint.setAntiAlias(true);
			linePaint.setStrokeWidth(lineWidth / 2);
		}
		canvas.drawLine(0, controlHeight / 2 - unitHeight / 2 + 2,
				controlWidth, controlHeight / 2 - unitHeight / 2 + 2, linePaint);
		canvas.drawLine(0, controlHeight / 2 + unitHeight / 2 - 2,
				controlWidth, controlHeight / 2 + unitHeight / 2 - 2, linePaint);

		if (bottomlinePaint == null) {
			bottomlinePaint = new Paint();
			bottomlinePaint.setColor(lineColor);
			bottomlinePaint.setAntiAlias(true);
			bottomlinePaint.setStrokeWidth(lineWidth);
		}
		canvas.drawLine(firstLineAndSecondLineSpace, controlHeight / 2
				- unitHeight / 2 + 2, controlWidth
				- firstLineAndSecondLineSpace, controlHeight / 2 - unitHeight
				/ 2 + 2, bottomlinePaint);
		canvas.drawLine(firstLineAndSecondLineSpace, controlHeight / 2
				+ unitHeight / 2 - 2, controlWidth
				- firstLineAndSecondLineSpace, controlHeight / 2 + unitHeight
				/ 2 - 2, bottomlinePaint);

	}

	/**
	 * ڸǰ
	 * 
	 * @param canvas
	 */
	private void drawMask(Canvas canvas) {
		LinearGradient lg = new LinearGradient(0, 0, 0, maskHight, 0x00f2f2f2,
				0x00f2f2f2, TileMode.MIRROR);
		Paint paint = new Paint();
		paint.setShader(lg);
		canvas.drawRect(0, 0, controlWidth, maskHight, paint);

		LinearGradient lg2 = new LinearGradient(0, controlHeight - maskHight,
				0, controlHeight, 0x00f2f2f2, 0x00f2f2f2, TileMode.MIRROR);
		Paint paint2 = new Paint();
		paint2.setShader(lg2);
		canvas.drawRect(0, controlHeight - maskHight, controlWidth,
				controlHeight, paint2);
	}

	/**
	 * ʼȡõ
	 * 
	 * @param context
	 * @param attrs
	 */
	private void init(Context context, AttributeSet attrs) {
		cachedThreadPool = Executors.newCachedThreadPool();
		TypedArray attribute = context.obtainStyledAttributes(attrs,
				R.styleable.WheelView);
		unitHeight = (int) attribute.getDimension(
				R.styleable.WheelView_unitHight, unitHeight);
		normalFont = attribute.getDimension(
				R.styleable.WheelView_normalTextSize, normalFont);
		selectedFont = attribute.getDimension(
				R.styleable.WheelView_selectedTextSize, selectedFont);
		itemNumber = attribute.getInt(R.styleable.WheelView_itemNumber,
				itemNumber);
		normalColor = attribute.getColor(R.styleable.WheelView_normalTextColor,
				normalColor);
		selectedColor = attribute.getColor(
				R.styleable.WheelView_selectedTextColor, selectedColor);
		lineColor = attribute.getColor(R.styleable.WheelView_lineColor,
				lineColor);
		bottomLineColor = attribute.getColor(
				R.styleable.WheelView_bottomLineColor, bottomLineColor);
		lineWidth = attribute.getDimension(R.styleable.WheelView_lineHeight,
				lineWidth);
		firstLineAndSecondLineSpace = attribute.getDimension(
				R.styleable.WheelView_firstLineAndSecondLineSpace,
				firstLineAndSecondLineSpace);
		maskHight = attribute.getDimension(R.styleable.WheelView_maskHight,
				maskHight);
		noEmpty = attribute.getBoolean(R.styleable.WheelView_noEmpty, true);
		isEnable = attribute.getBoolean(R.styleable.WheelView_isEnable, true);
		attribute.recycle();

		controlHeight = itemNumber * unitHeight;

	}

	/**
	 *  һΣ
	 * 
	 * @param data
	 */
	public void setData(ArrayList<String> data) {
		this.dataList = data;
		initData();
	}

	/**
	 * 
	 * 
	 * @param data
	 */
	public void resetData(ArrayList<String> data) {
		setData(data);
		invalidate();
	}

	/**
	 * ȡ id
	 * 
	 * @return
	 */
	public int getSelected() {
		for (ItemObject item : itemList) {
			if (item.isSelected())
				return item.id;
		}
		return -1;
	}

	/**
	 * ȡص
	 * 
	 * @return
	 */
	public String getSelectedText() {
		for (ItemObject item : itemList) {
			if (item.isSelected())
				return item.itemText;
		}
		return "";
	}

	/**
	 * Ƿڻ
	 * 
	 * @return
	 */
	public boolean isScrolling() {
		return isScrolling;
	}

	/**
	 * Ƿ
	 * 
	 * @return
	 */
	public boolean isEnable() {
		return isEnable;
	}

	/**
	 * Ƿ
	 * 
	 * @param isEnable
	 */
	public void setEnable(boolean isEnable) {
		this.isEnable = isEnable;
	}

	/**
	 * Ĭѡ
	 * 
	 * @param index
	 */
	public void setDefault(int index) {
		if (index > itemList.size() - 1)
			return;
		float move = itemList.get(index).moveToSelected();
		defaultMove((int) move);
	}

	/**
	 * ȡбС
	 * 
	 * @return
	 */
	public int getListSize() {
		if (itemList == null)
			return 0;
		return itemList.size();
	}

	/**
	 * ȡĳ
	 * 
	 * @param index
	 * @return
	 */
	public String getItemText(int index) {
		if (itemList == null)
			return "";
		return itemList.get(index).itemText;
	}

	/**
	 * 
	 * 
	 * @param onSelectListener
	 */
	public void setOnSelectListener(OnSelectListener onSelectListener) {
		this.onSelectListener = onSelectListener;
	}

	private void updateView() {
		mHandler.post(runnable);
	}

	Runnable runnable = new Runnable() {

		@Override
		public void run() {
			// TODO Auto-generated method stub
			invalidate();
		}
	};

	/**
	 * 
	 */
	private class ItemObject {
		/**
		 * id
		 */
		public int id = 0;
		/**
		 * 
		 */
		public String itemText = "";
		/**
		 * x
		 */
		public int x = 0;
		/**
		 * y
		 */
		public int y = 0;
		/**
		 * ƶ
		 */
		public int move = 0;
		/**
		 * 廭
		 */
		private Paint textPaint;
		/**
		 * 巶Χ
		 */
		private Rect textRect;

		public ItemObject() {
			super();
		}

		/**
		 * 
		 * 
		 * @param canvas
		 */
		public void drawSelf(Canvas canvas) {

			if (textPaint == null) {
				textPaint = new Paint();
				textPaint.setAntiAlias(true);
			}

			if (textRect == null)
				textRect = new Rect();

			// жǷѡ
			if (isSelected()) {
				textPaint.setColor(selectedColor);
				// ȡ׼λõľ
				float moveToSelect = moveToSelected();
				moveToSelect = moveToSelect > 0 ? moveToSelect : moveToSelect
						* (-1);
				// 㵱ǰС
				float textSize = (float) normalFont
						+ ((float) (selectedFont - normalFont) * (1.0f - (float) moveToSelect
								/ (float) unitHeight));
				textPaint.setTextSize(textSize);
			} else {
				textPaint.setColor(normalColor);
				textPaint.setTextSize(normalFont);
			}

			// ذΧַСһRect
			textPaint.getTextBounds(itemText, 0, itemText.length(), textRect);
			// жǷ
			if (!isInView())
				return;

			// 
			canvas.drawText(itemText, x + controlWidth / 2 - textRect.width()
					/ 2, y + move + unitHeight / 2 + textRect.height() / 2,
					textPaint);

		}

		/**
		 * Ƿڿӽ
		 * 
		 * @return
		 */
		public boolean isInView() {
			if (y + move > controlHeight
					|| (y + move + unitHeight / 2 + textRect.height() / 2) < 0)
				return false;
			return true;
		}

		/**
		 * ƶ
		 * 
		 * @param _move
		 */
		public void move(int _move) {
			this.move = _move;
		}

		/**
		 * µ
		 * 
		 * @param _move
		 */
		public void newY(int _move) {
			this.move = 0;
			this.y = y + _move;
		}

		/**
		 * жǷѡ
		 * 
		 * @return
		 */
		public boolean isSelected() {
			if ((y + move) >= controlHeight / 2 - unitHeight / 2 + 2
					&& (y + move) <= controlHeight / 2 + unitHeight / 2 - 2)
				return true;
			if ((y + move + unitHeight) >= controlHeight / 2 - unitHeight / 2
					+ 2
					&& (y + move + unitHeight) <= controlHeight / 2
							+ unitHeight / 2 - 2)
				return true;
			if ((y + move) <= controlHeight / 2 - unitHeight / 2 + 2
					&& (y + move + unitHeight) >= controlHeight / 2
							+ unitHeight / 2 - 2)
				return true;
			return false;
		}

		/**
		 * ȡƶ׼λҪľ
		 */
		public float moveToSelected() {
			return (controlHeight / 2 - unitHeight / 2) - (y + move);
		}
	}

	/**
	 * ѡ
	 */
	public interface OnSelectListener {
		/**
		 * ѡ
		 * 
		 * @param id
		 * @param text
		 */
		public void endSelect(int id, String text);

		/**
		 * ѡе
		 * 
		 * @param id
		 * @param text
		 */
		public void selecting(int id, String text);

	}
}
